@@ -23,6 +23,7 @@ from grid2viz.src.manager import (
 )
 from grid2viz.src.utils.callbacks_helpers import toggle_modal_helper
 from grid2viz.src.utils.common_graph import make_action_ts, make_rewards_ts
+from grid2viz.src.utils.graph_utils import layout_def, layout_no_data
 from grid2viz.src.utils.constants import DONT_SHOW_FILENAME
 from grid2viz.src.utils.graph_utils import (
     get_axis_relayout,
@@ -40,19 +41,22 @@ def register_callbacks_macro(app):
             Output("cumulated_rewards_timeserie", "figure"),
             Output("overflow_graph_study", "figure"),
             Output("usage_rate_graph_study", "figure"),
-            Output("action_timeserie", "figure"),
+            Output("action_topology_timeserie", "figure"),
+            Output("action_dispatch_timeserie", "figure"),
         ],
         [
             Input("agent_study", "modified_timestamp"),
             Input("agent_ref", "data"),
+
             Input("relayoutStoreMacro", "data"),
         ],
-        [
+        [   State("select_study_agent", "disabled"),
             State("rewards_timeserie", "figure"),
             State("cumulated_rewards_timeserie", "figure"),
             State("overflow_graph_study", "figure"),
             State("usage_rate_graph_study", "figure"),
-            State("action_timeserie", "figure"),
+            State("action_topology_timeserie", "figure"),
+            State("action_dispatch_timeserie", "figure"),
             State("scenario", "data"),
             State("agent_study", "data"),
             State("relayoutStoreMacro", "modified_timestamp"),
@@ -62,22 +66,28 @@ def register_callbacks_macro(app):
         agent_study_ts,
         ref_agent,
         relayout_data_store,
+        disabled,
         rew_figure,
         cumrew_figure,
         overflow_figure,
         usage_rate_figure,
-        actions_figure,
+        actions_topology_figure,
+        actions_dispatch_figure,
         scenario,
         study_agent,
         relayoutStoreMacro_ts,
     ):
 
+        if disabled :
+            raise PreventUpdate
+
         figures = [
             rew_figure,
             cumrew_figure,
             overflow_figure,
             usage_rate_figure,
-            actions_figure,
+            actions_topology_figure,
+            actions_dispatch_figure
         ]
 
         episode = make_episode(study_agent, scenario)
@@ -116,8 +126,8 @@ def register_callbacks_macro(app):
 
         usage_rate_figure["data"] = episode.usage_rate_trace
 
-        new_action_fig = make_action_ts(
-            study_agent, ref_agent, scenario, actions_figure["layout"]
+        new_topology_action_fig,new_dispatch_action_fig = make_action_ts(
+            study_agent, ref_agent, scenario, actions_topology_figure["layout"],actions_dispatch_figure
         )
 
         return (
@@ -125,49 +135,68 @@ def register_callbacks_macro(app):
             new_cumreward_fig,
             overflow_figure,
             usage_rate_figure,
-            new_action_fig,
+            new_topology_action_fig,
+            new_dispatch_action_fig
         )
 
     @app.callback(
         Output("agent_study_pie_chart", "figure"),
         [Input("agent_study", "data")],
-        [State("agent_study_pie_chart", "figure"), State("scenario", "data")],
+        [State("select_study_agent", "disabled"),
+        State("agent_study_pie_chart", "figure"), State("scenario", "data")],
     )
-    def update_action_repartition_pie(study_agent, figure, scenario):
+    def update_action_repartition_pie(study_agent,disabled, figure, scenario):
+        if disabled:
+            raise PreventUpdate
         new_episode = make_episode(study_agent, scenario)
-        figure["data"] = action_repartition_pie(new_episode)
-        figure["layout"].update(
-            actions_model.update_layout(
-                figure["data"][0].values == (0, 0, 0), "No Actions for this Agent"
-            )
-        )
+        figure = action_repartition_pie(new_episode)
+
         return figure
 
-    def action_repartition_pie(agent):
-        nb_actions = agent.action_data_table[
-            ["action_line", "action_subs", "action_redisp"]
-        ].sum()
-        return [
-            go.Pie(
-                labels=[
-                    "Actions on Lines",
-                    "Actions on Substations",
-                    "Redispatching Actions",
-                ],
-                values=[
-                    nb_actions["action_line"],
-                    nb_actions["action_subs"],
-                    nb_actions["action_redisp"],
+    def action_repartition_pie(episode):
+
+        actions_table = episode.action_data_table[
+            ["action_line", "action_subs", "action_redisp", "action_curtail","action_storage"]
+        ]
+        nb_actions = (actions_table > 0).sum()
+
+        if nb_actions.sum() == 0:
+            pie_figure = go.Figure(layout=layout_no_data("No Actions for this Agent"))
+        else:
+            pie_figure = go.Figure(
+                layout=layout_def,
+                data=[
+                    go.Pie(
+                        labels=[
+                            "Actions on Lines",
+                            "Actions on Substations",
+                            "Redispatching Actions",
+                            "Curtailment Actions",
+                            "Storage Actions"
+                        ],
+                        values=[
+                            nb_actions["action_line"],
+                            nb_actions["action_subs"],
+                            nb_actions["action_redisp"],
+                            nb_actions["action_curtail"],
+                            nb_actions["action_storage"]
+                        ],
+                        sort=False
+                    )
                 ],
             )
-        ]
+
+        return pie_figure
 
     @app.callback(
         Output("network_actions", "figure"),
         [Input("agent_study", "data")],
-        [State("scenario", "data")],
+        [State("select_study_agent", "disabled"),
+         State("scenario", "data")],
     )
-    def update_network_graph(study_agent, scenario):
+    def update_network_graph(study_agent,disabled, scenario):
+        if disabled:# is None:
+            raise PreventUpdate
         episode = make_episode(study_agent, scenario)
 
         return make_network_agent_overview(episode)
@@ -228,9 +257,13 @@ def register_callbacks_macro(app):
             Output("indicator_nb_action", "children"),
             Output("indicator_nb_maintenances", "children"),
         ],
-        [Input("agent_study", "data"), Input("scenario", "data")],
+        [
+         Input("agent_study", "data"), Input("scenario", "data")],
+        [State("select_study_agent", "disabled")]
     )
-    def update_nbs(study_agent, scenario):
+    def update_nbs(study_agent, scenario,disabled):
+        if disabled:
+            raise PreventUpdate
         new_episode = make_episode(study_agent, scenario)
         score = f"{get_score_agent(new_episode):,}"
         survival_time = (
@@ -282,10 +315,16 @@ def register_callbacks_macro(app):
         action_line="Action on line",
         action_subs="Action on sub",
         action_redisp="Action of redispatch",
+        action_curtail="Action of curtailment",
+        action_storage="Action of storage",
         redisp_impact="Redispatch impact",
+        curtail_impact="Curtailment impact",
+        storage_impact="Storage impact",
         line_name="Line name",
         sub_name="Sub name",
         gen_name="Gen name",
+        ren_name="Renewable plant name",
+        storage_name="Storage name",
         action_id="Action id",
         distance="Topological distance",
         is_alarm="is_alarm",
@@ -295,13 +334,18 @@ def register_callbacks_macro(app):
     @app.callback(
         [Output("inspector_datable", "columns"), Output("inspector_datable", "data")],
         [Input("agent_study", "data"), Input("scenario", "data")],
+        [State("select_study_agent", "disabled")],
+
     )
-    def update_agent_log_action_table(study_agent, scenario):
+    def update_agent_log_action_table(study_agent, scenario,disabled):
+        if disabled:# is None:
+            raise PreventUpdate
         new_episode = make_episode(study_agent, scenario)
         table = actions_model.get_action_table_data(new_episode)
         table["id"] = table["timestep"]
         table.set_index("id", inplace=True, drop=False)
-        cols_to_exclude = ["id", "lines_modified", "subs_modified", "gens_modified","is_action"]
+        cols_to_exclude = ["id","gen_name","ren_name","storage_name",
+                           "lines_modified", "subs_modified", "gens_modified","rens_modified","storages_modified","is_action"]
         cols = [
             {"name": action_table_name_converter[col], "id": col}
             for col in table.columns
@@ -317,6 +361,7 @@ def register_callbacks_macro(app):
         ],
         [Input("agent_study", "data"), Input("agent_ref", "data")],
         [
+            State("select_study_agent", "disabled"),
             State("distribution_substation_action_chart", "figure"),
             State("distribution_line_action_chart", "figure"),
             State("distribution_redisp_action_chart", "figure"),
@@ -324,8 +369,10 @@ def register_callbacks_macro(app):
         ],
     )
     def update_agent_log_action_graphs(
-        study_agent, ref_agent, figure_sub, figure_switch_line, figure_redisp, scenario
+        study_agent, ref_agent, disabled, figure_sub, figure_switch_line, figure_redisp, scenario
     ):
+        if disabled:# is None:
+            raise PreventUpdate
         new_episode = make_episode(study_agent, scenario)
         ref_episode = make_episode(ref_agent, scenario)
         y_max = None
