@@ -17,6 +17,7 @@ import matplotlib.pyplot as plt
 import numpy as np
 import pandas as pd
 import plotly.graph_objects as go
+from scipy.spatial import ConvexHull
 from grid2op.Episode import EpisodeData
 from grid2op.PlotGrid import PlotPlotly, PlotMatplot
 
@@ -69,6 +70,32 @@ def add_substation_color_plotly(subs, plot_helper, fig, color="gold"):
         )
     return fig
 
+def add_alarm_area_plotly(line_subs, plot_helper, fig, color="gold"):
+
+    x=[]
+    y=[]
+    for id_sub in line_subs:
+        subName = "sub_" + str(id_sub)
+        x_center, y_center = plot_helper._grid_layout[subName]
+        x.append(x_center)
+        y.append(y_center)
+
+    points = [[lx, ly] for lx, ly in zip(x, y)]
+    hull = ConvexHull(points)
+    hull_vertices_x = [x[i] for i in hull.vertices]
+    hull_vertices_y = [y[i] for i in hull.vertices]
+    fig.add_trace(go.Scatter(
+        x=hull_vertices_x,
+        y=hull_vertices_y,
+        marker=dict(color=color, size=2),
+        mode="markers",
+        # name="Women",
+        fill="toself",
+        opacity=0.5
+    ))
+    return fig
+
+
 
 def make_network(episode, responsive=True):
     """
@@ -87,7 +114,7 @@ def make_network(episode, responsive=True):
     return graph
 
 
-def make_network_matplotlib(episode):
+def make_network_matplotlib(episode,timestep=0):
     global graph_matplotlib
     if graph_matplotlib is None:
         graph_matplotlib = PlotMatplot(
@@ -103,17 +130,96 @@ def make_network_matplotlib(episode):
 ######
 # we want a non responsive graph for now in agent_study
 # so we have to define it differently from the global graph in make_network that we don't use here
-def make_network_agent_study(episode, timestep, responsive=False):
+import base64
+import io
+def make_network_agent_study(episode, timestep, figure_obs=None, responsive=False,redraw=False):
     # subs_on_bus_2 = np.repeat(False, episode_data.observations[0].n_sub)
-    graph = PlotPlotly(
-        grid_layout=episode.observation_space.grid_layout,
-        observation_space=episode.observation_space,
-        responsive=responsive,
-    )
+    #graph=None
+    #if(isMatplotLib):########not working for now. Was trying to use matplotlib to accelerate ploting time
+     #   buf = io.BytesIO()
+     #   make_network_scenario_overview(episode,timestep=timestep)
+#
+#
+     #   # plt.figure(network_graph.number)
+     #   # plt.close(fig)
+     #   plt.savefig(buf, format="png")
+     #   buf.seek(0)
+     #   #encoded_image = base64.b64encode(buf.read())
+#
+     #   #fig=encoded_image.decode()
+     #   data = base64.b64encode(buf.getbuffer()).decode("utf8")  # encode to html elements
+     #   buf.close()
+     #   return "data:image/png;base64,{}".format(data)
+
+    observation=episode.observations[timestep]
+
+    graph=make_network(episode, responsive)
     graph._sub_radius = 30  # instead of 25 by default
     graph._bus_radius = 10  # instead of 4 by default
+    if(figure_obs)and not redraw:# don't redraw it from scratch, just change what is needed
+
+        import plotly.colors as pc
+        data_fig=figure_obs["data"]#go.Figure(figure_obs)
+
+        rho_lines=observation.rho
+        n_lines=len(rho_lines)
+
+        id_line=0
+        i_traces=0
+        previous_trace=None
+        while id_line<n_lines:
+            trace=data_fig[i_traces]
+
+            if "line" in trace.keys():#update line color
+                color_scheme = (
+                        pc.sequential.Blues_r[:4]
+                        + pc.sequential.Oranges[4:6]
+                        + pc.sequential.Reds[-3:-1]
+                )
+                rho=rho_lines[id_line]
+                capacity = np.clip(rho, 0.0, 1.0)
+                color = color_scheme[int(capacity * float(len(color_scheme) - 1))]
+                trace["line"]['color']=str(color)
+                if(previous_trace) and ("text" in previous_trace.keys()):
+                    previous_trace["text"]=[str(np.round(rho*100,2))+" %"]
+
+                id_line+=1
+            i_traces+=1
+            previous_trace=trace
+
+        #remove contextual trace such as color sub for specific topology and alarm zone
+
+        #remove alarm first
+        n_traces=len(data_fig)-1
+        for i_traces in range(n_traces,-1,-1):
+            trace = data_fig[i_traces]
+            if "fill" in trace.keys():
+                data_fig.pop(i_traces)
+            elif "sub" in str(trace["text"]):
+                break
+
+        #update topology traces at substations
+        n_traces=len(data_fig)-1
+        for i_traces in range(n_traces,-1,-1):
+            trace = data_fig[i_traces]
+            if "sub" in str(trace["text"]):
+                if "marker" in trace.keys():
+                    if trace["marker"]["color"]!='PaleTurquoise':
+                        data_fig.pop(i_traces)
+                    else:
+                        break
+
+        fig=go.Figure(figure_obs)
+
+    else:
+        #if graph is None:
+        #    graph = PlotPlotly(
+        #        grid_layout=episode.observation_space.grid_layout,
+        #        observation_space=episode.observation_space,
+        #        responsive=responsive,
+        #    )
+        fig = graph.plot_obs(episode.observations[timestep])#,redraw=redraw,figure=figure_obs)
 
-    fig = graph.plot_obs(episode.observations[timestep])
 
     ##########
     # We color subs where we had actions
@@ -124,7 +230,7 @@ def make_network_agent_study(episode, timestep, responsive=False):
         int(str.split("_")[1])
         for str in episode.action_data_table.subs_modified[timestep]
     ]
-    fig = add_substation_color_plotly(sub_id_modified, graph, fig)
+    fig = add_substation_color_plotly(sub_id_modified, graph,fig)
 
     # coloring subs not in reference topologie
     nb_bus_subs = [
@@ -137,6 +243,30 @@ def make_network_agent_study(episode, timestep, responsive=False):
     fig = add_substation_color_plotly(
         sub_2buses, graph, fig, color="green"
     )  # also other color for subs not in ref topo
+
+    if ("is_alarm" in episode.action_data_table.columns):
+        alarms_lines_area = episode.observations[timestep].alarms_lines_area
+
+        light_colors_plotly = ["lightcoral", "lightsalmon", "lightpink"]
+        n_colors = len(light_colors_plotly)
+
+        if (episode.action_data_table.is_alarm[timestep]):
+
+            alarm_zones = episode.action_data_table.alarm_zone[timestep]
+
+            for i_zone,zone in enumerate(alarm_zones):
+                id_lines_alarm = []
+                for idx, line_name in enumerate(episode.observations[timestep].name_line):
+                    line_alarm_zones = alarms_lines_area[line_name]
+                    if(zone in line_alarm_zones):
+                        id_lines_alarm.append(idx)
+                line_subs = [episode.observations[timestep].line_ex_to_subid[l_idx] for l_idx in id_lines_alarm]
+                line_subs += [episode.observations[timestep].line_or_to_subid[l_idx] for l_idx in id_lines_alarm]
+                line_subs = np.unique(line_subs)
+
+                area_color= i_zone % n_colors
+                fig = add_alarm_area_plotly(line_subs, graph, fig, color=light_colors_plotly[area_color])
+
     return fig
 
 
@@ -194,7 +324,7 @@ def make_network_agent_overview(episode):
     return fig
 
 
-def make_network_scenario_overview(episode):
+def make_network_scenario_overview(episode,timestep=0):
     max_loads = (
         episode.load[["value", "equipement_id"]]
         .groupby("equipement_id")
@@ -217,7 +347,7 @@ def make_network_scenario_overview(episode):
     # we will use plot_obs instead of plot_info for now
     ####
     # For that we override an observation with the desired values
-    obs_colored = episode.observations[0]
+    obs_colored = episode.observations[timestep]
 
     # having a rho with value 0.1 give us a blue line while 0.5 gives us an orange line
     # line in maintenance would display as dashed lines
@@ -267,6 +397,10 @@ def make_episode(agent, episode_name,with_reboot=False):
     elif is_in_fs_cache(episode_name, agent):
         episode = get_from_fs_cache(episode_name, agent)
         save_in_ram_cache(episode_name, agent, episode)
+        #to see evolution of ram footprint
+        #from guppy import hpy
+        #h = hpy()
+        #print(h.heap())
     else:
         episode = compute_episode(episode_name, agent,with_reboot)
         save_in_ram_cache(episode_name, agent, episode)
@@ -290,8 +424,12 @@ def make_episode_without_decorate(agent, episode_name,save=False):
     :return: Episode with computed data (without EpisodeData attributes), EpisodeData instance
     """
     if is_in_ram_cache(episode_name, agent):
+        if save:
+            return None
         return get_from_ram_cache(episode_name, agent)
     elif is_in_fs_cache(episode_name, agent):
+        if save:
+            return None
         beg = time.time()
         episode_analytics=get_from_fs_cache(episode_name, agent)
         return episode_analytics
@@ -359,6 +497,7 @@ def get_from_fs_cache(episode_name, agent):
 
         with gzip.open(path + ".bz", "rb") as f:
             # with zipfile.ZipFile.open(path + ".zip") as f:
+            print(path)
             episode_analytics=pickle.load(f)
     else:
         with open(path, "rb") as f:
@@ -375,7 +514,8 @@ def get_from_fs_cache(episode_name, agent):
     if("observations" not in dir(episode_analytics)):
         print("WARNING: the cache management have been updated in grid2viz 1.3.1 for faster loading. "
               "You Should delete the old _cache folder and recompute it with latest grid2viz version")
-    episode_analytics.optimize_memory_footprint()
+    episode_analytics.optimize_memory_footprint(opt_obs_act=True)#this adds a bit of 25% loading time overhead,
+    # in particular when resetting observations and actions, which only brings a 10% size decrease
 
     #episode_analytics.decorate(episode_data)
     #episode_analytics=decorate(episode_analytics,episode_data)
@@ -539,7 +679,7 @@ def make_cache(scenarios,agents,n_cores,cache_dir,agent_selection=None):
             i += 1
     else:
         pool = ProcessPool(n_cores)
-        agents_data = list(
+        list(
             pool.imap(
                 make_episode_without_decorate,
                 [agent_scenario[0] for agent_scenario in agent_scenario_list],  # agents
