@@ -5,6 +5,7 @@
 import datetime as dt
 
 import dash_core_components as dcc
+from dash import callback_context
 import plotly.graph_objects as go
 from dash.dependencies import Input, Output, State
 from dash.exceptions import PreventUpdate
@@ -24,6 +25,7 @@ from grid2viz.src.utils.graph_utils import (
 
 
 def register_callbacks_micro(app):
+
     @app.callback(
         [
             Output("slider", "min"),
@@ -31,33 +33,99 @@ def register_callbacks_micro(app):
             Output("slider", "value"),
             Output("slider", "marks"),
         ],
-        [Input("window", "data")],
+        [Input("window", "data"),Input("url", "pathname"),Input('auto-stepper', 'n_intervals'),Input("card-tabs", "active_tab")], #Input("card-tabs", "active_tab")
         [
-            State("slider", "value"),
+            State('my-toggle-switch', "on"),
+            #State("slider", "min"),
+            #State("slider", "max"),
+            #State("slider", "marks"),
             State("user_timestamps", "value"),
             State("agent_study", "data"),
+            State("agent_ref", "data"),
             State("scenario", "data"),
         ],
     )
-    def update_slider(window, value, selected_timestamp, study_agent, scenario):
+    def update_slider(window,url,n_intervals,active_tab,togle_value, selected_timestamp, agent_study,agent_ref, scenario):
         if window is None:
             raise PreventUpdate
-        new_episode = make_episode(study_agent, scenario)
+        if(type(url) is not str):
+            raise PreventUpdate
+        url_split = url.split("/")
+        url_split = url_split[len(url_split) - 1]
 
+        if(url_split!="micro"):
+            raise PreventUpdate
+
+        ctx = callback_context
+        # No clicks
+        if not ctx.triggered:
+            raise PreventUpdate
+        # No clicks again
+        # https://github.com/plotly/dash/issues/684
+        if ctx.triggered[0]["value"] is None:
+            raise PreventUpdate
+
+        button_id = ctx.triggered[0]['prop_id'].split('.')[0]
+
+        new_episode = make_episode(
+            agent_study if active_tab == "tab-0" else agent_ref, scenario
+        )
         min_ = new_episode.timestamps.index(
             dt.datetime.strptime(window[0], "%Y-%m-%dT%H:%M:%S")
         )
         max_ = new_episode.timestamps.index(
             dt.datetime.strptime(window[1], "%Y-%m-%dT%H:%M:%S")
         )
-        if value not in range(min_, max_):
+        if(button_id == "auto-stepper"):
+            if not togle_value:
+                raise PreventUpdate
+            else:
+                value=min_+(n_intervals)%(max_-min_)
+
+        else:
+
+            #if value not in range(min_, max_):
             value = new_episode.timestamps.index(
                 dt.datetime.strptime(selected_timestamp, "%Y-%m-%d %H:%M")
             )
+        timestamp_range = new_episode.timestamps[min_:(max_+1)]
+        timestamp_range = [timestamp.time() for timestamp in timestamp_range]
 
-        marks = dict(enumerate(map(lambda x: x.time(), new_episode.timestamps)))
+        if("is_action" in new_episode.action_data_table.columns):
+            is_actions = list(new_episode.action_data_table.is_action[min_:(max_+1)].values)
+
+            marks = {int(min_ + idx): {'label': t, 'style': {'color': 'orange'}} if is_act else {'label': t, 'style': {
+                'color': 'black'}} for idx, (t, is_act) in enumerate(zip(timestamp_range, is_actions))}
+
+            if ("is_alarm" in new_episode.action_data_table.columns):
+                is_alarm = list(new_episode.action_data_table.is_alarm[min_:(max_ + 1)].values)
+                for idx, mark_key in enumerate(marks.keys()):
+                    if is_alarm[idx]:
+                        marks[mark_key]['style']['background-color']='lightcoral'
+        else:
+            marks = {int(min_ + idx): {'label': t, 'style': {'color': 'black'}} for idx, t in enumerate(timestamp_range)}
 
         return min_, max_, value, marks
+#
+
+    @app.callback(
+        [
+            Output('auto-stepper', 'interval'),
+            Output('auto-stepper', 'n_intervals'),
+        ],
+        Input('my-toggle-switch', "on"),
+        State('auto-stepper', 'n_intervals'),
+    )
+    def activate_stepper(value,n_intervals):
+#
+        if(value):
+            #disable_stepper=False
+            interval=2750
+            return interval,n_intervals
+        else:
+            disable_stepper=True
+            interval = 30000000
+            return interval,0
 
     @app.callback(
         Output("relayoutStoreMicro", "data"),
@@ -67,7 +135,8 @@ def register_callbacks_micro(app):
             Input("overflow_ts", "relayoutData"),
             Input("rewards_ts", "relayoutData"),
             Input("cumulated_rewards_ts", "relayoutData"),
-            Input("actions_ts", "relayoutData"),
+            Input("action_topology_ts", "relayoutData"),
+            Input("action_dispatch_ts", "relayoutData"),
             Input("voltage_flow_graph", "relayoutData"),
         ],
         [State("relayoutStoreMicro", "data")],
@@ -80,7 +149,8 @@ def register_callbacks_micro(app):
         [
             Output("rewards_ts", "figure"),
             Output("cumulated_rewards_ts", "figure"),
-            Output("actions_ts", "figure"),
+            Output("action_topology_ts", "figure"),
+            Output("action_dispatch_ts", "figure"),
         ],
         [
             Input("relayoutStoreMicro", "data"),
@@ -90,7 +160,8 @@ def register_callbacks_micro(app):
         [
             State("rewards_ts", "figure"),
             State("cumulated_rewards_ts", "figure"),
-            State("actions_ts", "figure"),
+            State("action_topology_ts", "figure"),
+            State("action_dispatch_ts", "figure"),
             State("agent_ref", "data"),
             State("scenario", "data"),
         ],
@@ -101,12 +172,13 @@ def register_callbacks_micro(app):
         study_agent,
         rew_figure,
         cumrew_figure,
-        actions_figure,
+        topology_action_fig,
+        dispatch_action_fig,
         agent_ref,
         scenario,
     ):
 
-        figures = [rew_figure, cumrew_figure, actions_figure]
+        figures = [rew_figure, cumrew_figure, topology_action_fig,dispatch_action_fig]
 
         condition = (
             relayout_data_store is not None and relayout_data_store["relayout_data"]
@@ -126,11 +198,11 @@ def register_callbacks_micro(app):
             study_agent, agent_ref, scenario, rew_figure, cumrew_figure
         )
 
-        actions_figure = common_graph.make_action_ts(
-            study_agent, agent_ref, scenario, actions_figure["layout"]
+        new_topology_action_fig,new_dispatch_action_fig = common_graph.make_action_ts(
+            study_agent, agent_ref, scenario, topology_action_fig["layout"]
         )
-
-        figures = [rew_figure, cumrew_figure, actions_figure]
+        #TO DO
+        figures = [rew_figure, cumrew_figure, new_topology_action_fig,new_dispatch_action_fig]
 
         if window is not None:
             start_datetime = dt.datetime.strptime(window[0], "%Y-%m-%dT%H:%M:%S")
@@ -408,18 +480,24 @@ def register_callbacks_micro(app):
 
     @app.callback(
         [
-            Output("card-content", "children"),
+            #Output("card-content", "children"),
+            Output("interactive_graph", "figure"),
             Output("tooltip_table_micro", "children"),
+            Output("slider","disabled"),
+            Output("my-toggle-switch", "disabled"),
         ],
-        [Input("slider", "value"), Input("card-tabs", "active_tab")],
+        [Input("slider", "value"), Input("card-tabs", "active_tab")],Input("my-toggle-switch", "off"),
         [
+
             State("agent_study", "data"),
             State("scenario", "data"),
             State("agent_ref", "data"),
+            State("interactive_graph", "figure"),
+            #State("card-content", "children"),
         ],
     )
     def update_interactive_graph(
-        slider_value, active_tab, agent_study, scenario, agent_ref
+        slider_value, active_tab,Power_Button_off, agent_study, scenario, agent_ref,current_network_fig,#,card_content#,
     ):
         episode = make_episode(
             agent_study if active_tab == "tab-0" else agent_ref, scenario
@@ -427,7 +505,9 @@ def register_callbacks_micro(app):
 
         try:
             act = episode.actions[slider_value]
-        except Grid2OpException as ex:
+        except:# Grid2OpException as ex:
+            disabled_Power_Button=True
+            disabled_Slider=True
             return (
                 dcc.Graph(
                     figure=go.Figure(
@@ -437,14 +517,26 @@ def register_callbacks_micro(app):
                     )
                 ),
                 "",
+                disabled_Slider,
+                disabled_Power_Button
             )
+        #current_network_fig=None#card_content[0]['props']['figure']#None
+        if(current_network_fig):
+            print("fig here")
+        disabled_Power_Button = False
+        disabled_Slider = False
+        redraw_full_graph = not bool(Power_Button_off)
         if any(act.get_types()):
             act_as_str = str(act)
         else:
             act_as_str = "NO ACTION"
         return (
-            dcc.Graph(figure=make_network_agent_study(episode, timestep=slider_value)),
+            #dcc.Graph(figure=make_network_agent_study(episode, timestep=slider_value,figure_obs=current_network_fig)),
+            #yes in case the power button is turned off
+            make_network_agent_study(episode, timestep=slider_value, figure_obs=current_network_fig,redraw=redraw_full_graph),
             act_as_str,
+            disabled_Slider,
+            disabled_Power_Button
         )
 
     @app.callback(
@@ -468,4 +560,4 @@ def register_callbacks_micro(app):
 
     @app.callback(Output("modal_image_micro", "src"), [Input("url", "pathname")])
     def show_image(pathname):
-        return app.get_asset_url("screenshots/agent_study.png")
+        return app.get_asset_url("screenshots/agent_study.png")
\ No newline at end of file
